%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Generalized Learning Vector Quantizer - simple version
%
% (C) Frank-Michael Schleif (c) 2014
% University of Birmingham
% schleify@cs.bham.ac.uk
%
%
% Input:
% 	      Data              (N x d)  data matrix with N objects measured with d features each
%         vLabels    		(N x 1)  a numerical vector of class lables (0,0,0,1,1,1,...,2,2,2 ...)
%         optional          	see demo script
%
% Output:
%        oModel			
%
% This is test code only - some features are off/on see conditions in the
% code, some others can be defined by the parameter object
%
% See sample code how to use this algorithm and how to apply the model to new data
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [oModel] = glvq_train(Data,vLabels,iNrOfPrototypes,varargin)    
	%randreset(1234)
    vLabels = vLabels(:);
    opts.LS = 2;
    opts.LS_init = 1;
    opts.MaxIter = 100;
    opts.Method = 'sd';
    opts.Display = 'final';
    iMode = 2;
    iDimM = 2;
    if(~isempty(varargin))
        iMode = varargin{1};
        iDimM = varargin{2};
    end
    
    vUniqueLabels = unique(vLabels);
    C             = numel(vUniqueLabels);    
    [N,D]       = size(Data);    
    if(0)
        Neurons     = rand(iNrOfPrototypes,D);
        for(k=1:iNrOfPrototypes)
            vCodeLabels(k) = vUniqueLabels(mod(k,C)+1);
        end        
    elseif(0)
        [idx, Neurons, iNrOfPrototypes] = eff_kmeans(Data, iNrOfPrototypes, 100);
        for(k=1:iNrOfPrototypes)
           vCodeLabels(k) = mode(vLabels(idx==k)); 
        end
    else
        Neurons = zeros(iNrOfPrototypes,D);
        iCounter = 1;
        while(iCounter <= iNrOfPrototypes)
            for(k=1:numel(vUniqueLabels))
                vCandidates = find(vLabels==vUniqueLabels(k));
                iSelectedCandiate = randsample(numel(vCandidates),1);
                Neurons(iCounter,:)      = Data(vCandidates(iSelectedCandiate),:);
                vCodeLabels(iCounter)    = vUniqueLabels(k);
                iCounter = iCounter+1;
            end
        end
    end
    LabelEqualsPrototype = bsxfun(@eq,vLabels,vCodeLabels);
    InitMatrix = rand(D,iDimM); % Lambda = R*R';
    
    switch iMode
        case 0
            fDistPtr        = EuclideanDistance(D);
        case 1
            fDistPtr        = ParametricDistance(D);    
            fDistPtr.SetParameter(ones(D,1)./D);              
        case 2
            fDistPtr        = ParametricMatrixDistance(D,iDimM);             
            fDistPtr.SetParameter(InitMatrix');      
    end
    
    for(k=1:10)
        if(k==2) % first run without metric adaptation
            variables           = [variables;fDistPtr.GetParameter()];
        else
            variables           = [Neurons(:)];
        end
        fCostFct            = @(variables)glvq_optfun(variables,Data,fDistPtr,LabelEqualsPrototype,{vLabels,vCodeLabels});     
        variables           = minFunc(fCostFct,variables,opts);       
    end
    oModel.vCodeLabels  = vCodeLabels;
    oModel.Neurons      = reshape(variables(1:iNrOfPrototypes*D),iNrOfPrototypes,D);
    oModel.fDist        = fDistPtr;
    estimatedLabels     = glvq_classify(Data, oModel);
    oModel.TrainAcc     = mean(vLabels(:) == estimatedLabels(:))

   